@@ -24,8 +24,10 @@
 #include <fstream>
 #include <ios>
 #include <istream>
+#include <iterator>
 #include <limits>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <stdexcept>
 #include <string>
@@ -53,8 +55,8 @@ int32_t decodeValue(V raw) {
 
 template <typename T>
 struct NanValue {
-    static inline int32_t NO_VALUE() { return -1; };
-    static inline int32_t NO_PATH() { return -2; }
+    static int32_t NO_VALUE() { return -1; };
+    static int32_t NO_PATH() { return -2; }
 };
 
 // Musl doesn't have nanf implementation we need, just check if they are the
@@ -69,16 +71,16 @@ template <>
 struct NanValue<float> {
     static_assert(std::numeric_limits<float>::has_quiet_NaN,
                   "Require support for quiet NaN.");
-    static inline int32_t NO_VALUE() {
+    static int32_t NO_VALUE() {
         return isGoodNanf() ? decodeValue(std::nanf("1")) : 0x7fc00001;
     }
-    static inline int32_t NO_PATH() {
+    static int32_t NO_PATH() {
         return isGoodNanf() ? decodeValue(std::nanf("2")) : 0x7fc00002;
     }
 };
 
 template <typename T>
-constexpr inline T decodeImpl(int32_t raw) {
+constexpr T decodeImpl(int32_t raw) {
     typename DATriePrivate<T>::decoder_type d;
     d.result = raw;
     return d.result_value;
@@ -86,13 +88,12 @@ constexpr inline T decodeImpl(int32_t raw) {
 
 } // namespace
 
-#if 0
-template<typename T>
-using vector_impl = std::vector<T>;
-#else
+// template<typename T>
+// using vector_impl = std::vector<T>;
+// naivevector is used due to realloc not available with std::vector.
 template <typename T>
 using vector_impl = naivevector<T>;
-#endif
+
 template <typename V, bool ORDERED, int MAX_TRIAL>
 class DATriePrivate {
 public:
@@ -106,7 +107,8 @@ public:
     static inline const int32_t CEDAR_NO_VALUE = NanValue<V>::NO_VALUE();
     static inline const int32_t CEDAR_NO_PATH = NanValue<V>::NO_PATH();
 
-    static constexpr int MAX_ALLOC_SIZE = 1 << 16; // must be divisible by 256
+    static constexpr size_t MAX_ALLOC_SIZE = 1
+                                             << 16; // must be divisible by 256
     using result_type = value_type;
     using uchar = uint8_t;
     static_assert(sizeof(value_type) <= sizeof(int32_t),
@@ -361,20 +363,20 @@ public:
     }
 
     template <typename U>
-    inline void update(const char *key, const U &callback) {
+    void update(const char *key, const U &callback) {
         update(key, std::strlen(key), callback);
     }
 
     template <typename U>
-    inline void update(const char *key, size_t len, const U &callback) {
+    void update(const char *key, size_t len, const U &callback) {
         npos_t from;
         size_t pos(0);
         update(key, from, pos, len, callback);
     }
 
     template <typename U>
-    inline void update(const char *key, npos_t &from, size_t &pos, size_t len,
-                       const U &callback) {
+    void update(const char *key, npos_t &from, size_t &pos, size_t len,
+                const U &callback) {
         update(key, from, pos, len, callback, [](const int, const int) {});
     }
 
@@ -481,10 +483,7 @@ public:
         if (m_tail.capacity() < m_tail.size() + needed) {
             auto quota =
                 m_tail.capacity() +
-                ((needed > m_tail.size() || needed > MAX_ALLOC_SIZE)
-                     ? needed
-                     : (m_tail.size() >= MAX_ALLOC_SIZE ? MAX_ALLOC_SIZE
-                                                        : m_tail.size()));
+                std::max(needed, std::min(MAX_ALLOC_SIZE, m_tail.size()));
             m_tail.reserve(quota);
         }
         m_array[from].base = -m_tail.size();
@@ -560,7 +559,7 @@ public:
     void shrink_tail() {
         const size_t length_ =
             static_cast<size_t>(m_tail.size()) -
-            static_cast<size_t>(m_tail0.size()) * (1 + sizeof(value_type));
+            (static_cast<size_t>(m_tail0.size()) * (1 + sizeof(value_type)));
         decltype(m_tail) t;
         // a dummy entry
         t.resize(sizeof(int32_t));
@@ -592,8 +591,11 @@ public:
             npos.offset ? -static_cast<int>(npos.offset) : m_array[from].base;
         if (base >= 0) { // on trie
             uchar c = m_ninfo[from].child;
-            if (!from && !(c = m_ninfo[base ^ c].sibling)) { // bug fix
-                return CEDAR_NO_PATH;                        // no entry
+            if (!from) {
+                c = m_ninfo[base ^ c].sibling;
+                if (!c) {                 // bug fix
+                    return CEDAR_NO_PATH; // no entry
+                }
             }
             for (; c && base >= 0; ++len) {
                 from = static_cast<size_t>(base) ^ c;
@@ -638,11 +640,13 @@ public:
     int _follow(uint32_t &from, const uchar label, const T &cf) {
         int to = 0;
         const int base = m_array[from].base;
-        if (base < 0 || m_array[to = base ^ label].check < 0) {
+        if (base < 0 || m_array[base ^ label].check < 0) {
             to = _pop_enode(base, label, static_cast<int>(from));
             _push_sibling(from, to ^ label, label, base >= 0);
-        } else if (m_array[to].check != static_cast<int>(from)) {
+        } else if (m_array[base ^ label].check != static_cast<int>(from)) {
             to = _resolve(from, base, label, cf);
+        } else {
+            to = base ^ label;
         }
         return to;
     }
@@ -707,7 +711,7 @@ public:
     int _find_place(const uchar *const first, const uchar *const last) {
         if (auto bi = m_bheadO) {
             const auto bz = m_block[m_bheadO].prev;
-            const auto nc = static_cast<short>(last - first + 1);
+            const auto nc = std::distance(first, last);
             while (true) { // set candidate block
                 block &b = m_block[bi];
                 if (b.num >= nc && nc < b.reject) { // explore configuration
@@ -715,11 +719,13 @@ public:
                         const int base = e ^ *first;
                         for (const uchar *p = first;
                              m_array[base ^ *++p].check < 0;) {
-                            if (p == last) {
-                                return b.ehead = e; // no conflict
+                            if (p + 1 == last) {
+                                b.ehead = e;
+                                return b.ehead; // no conflict
                             }
                         }
-                        if ((e = -m_array[e].check) == b.ehead) {
+                        e = -m_array[e].check;
+                        if (e == b.ehead) {
                             break;
                         }
                     }
@@ -741,8 +747,8 @@ public:
         return _add_block() << 8;
     }
 
-    static void _set_result(result_type *x, value_type r, size_t = 0,
-                            npos_t = npos_t()) {
+    static void _set_result(result_type *x, value_type r, size_t /*len*/ = 0,
+                            npos_t /*npos*/ = npos_t()) {
         *x = r;
     }
     static void _set_result(std::tuple<value_type, size_t, position_type> *x,
@@ -892,23 +898,27 @@ public:
         return c_p;
     }
     // enumerate (equal to or more than one) child nodes
-    uchar *_set_child(uchar *p, const int base, uchar c, const int label = -1) {
-        --p;
+    uchar *_set_child(uchar *p, const int base, uchar c,
+                      const std::optional<uchar> label = std::nullopt) {
         if (!c) {
-            *++p = c;
+            *p = c;
+            p++;
             c = m_ninfo[base ^ c].sibling;
         } // 0: terminal
-        if (ORDERED) {
-            while (c && c < label) {
-                *++p = c;
+        if (ORDERED && label.has_value()) {
+            while (c && c < *label) {
+                *p = c;
+                p++;
                 c = m_ninfo[base ^ c].sibling;
             }
         }
-        if (label != -1) {
-            *++p = static_cast<uchar>(label);
+        if (label.has_value()) {
+            *p = *label;
+            p++;
         }
         while (c) {
-            *++p = c;
+            *p = c;
+            p++;
             c = m_ninfo[base ^ c].sibling;
         }
         return p;
@@ -925,30 +935,33 @@ public:
             = _consult(base_n, base_p, m_ninfo[from_n].child,
                        m_ninfo[from_p].child);
         uchar child[256];
-        uchar *const first = &child[0];
+        uchar *const first = child;
         uchar *const last =
             flag ? _set_child(first, base_n, m_ninfo[from_n].child, label_n)
                  : _set_child(first, base_p, m_ninfo[from_p].child);
+        assert(first < last);
         const int base =
-            (first == last ? _find_place() : _find_place(first, last)) ^ *first;
+            (first + 1 == last ? _find_place() : _find_place(first, last)) ^
+            *first;
         // replace & modify empty list
         const int from = flag ? static_cast<int>(from_n) : from_p;
         const int base_ = flag ? base_n : base_p;
         if (flag && *first == label_n) {
             m_ninfo[from].child = label_n; // new child
         }
-        m_array[from].base = base;                     // new base
-        for (const uchar *p = first; p <= last; ++p) { // to_ => to
+        m_array[from].base = base;                    // new base
+        for (const uchar *p = first; p < last; ++p) { // to_ => to
             const int to = _pop_enode(base, *p, from);
             const int to_ = base_ ^ *p;
-            m_ninfo[to].sibling = (p == last ? 0 : *(p + 1));
+            m_ninfo[to].sibling = (p + 1 == last) ? 0 : *(p + 1);
             if (flag && to_ == to_pn) {
                 continue; // skip newcomer (no child)
             }
             cf(to_, to);
             node &n = m_array[to];
             node &n_ = m_array[to_];
-            if ((n.base = n_.base) > 0 && *p) { // copy base; bug fix
+            n.base = n_.base; // copy base; bug fix
+            if (n.base > 0 && *p) {
                 uchar c = m_ninfo[to].child = m_ninfo[to_].child;
                 do {
                     m_array[n.base ^ c].check = to; // adjust grand son's check
@@ -1217,13 +1230,13 @@ size_t DATrie<T>::mem_size() const {
     //               << "ninfo" << sizeof(typename
     //               decltype(d->m_ninfo)::value_type) *
     //               d->m_ninfo.size() << std::endl;
-    return d->m_tail.size() + d->m_tail0.size() * sizeof(int) +
-           sizeof(typename decltype(d->m_array)::value_type) *
-               d->m_array.size() +
-           sizeof(typename decltype(d->m_block)::value_type) *
-               d->m_block.size() +
-           sizeof(typename decltype(d->m_ninfo)::value_type) *
-               d->m_ninfo.size();
+    return d->m_tail.size() + (d->m_tail0.size() * sizeof(int)) +
+           (sizeof(typename decltype(d->m_array)::value_type) *
+            d->m_array.size()) +
+           (sizeof(typename decltype(d->m_block)::value_type) *
+            d->m_block.size()) +
+           (sizeof(typename decltype(d->m_ninfo)::value_type) *
+            d->m_ninfo.size());
 }
 
 template class DATrie<float>;
