@@ -6,15 +6,15 @@
 
 #include "decoder.h"
 #include "languagemodel.h"
+#include "lattice.h"
 #include "lattice_p.h"
-#include "libime/core/lattice.h"
-#include "libime/core/segmentgraph.h"
+#include "segmentgraph.h"
 #include "utils.h"
+#include "utils_p.h"
 #include <algorithm>
 #include <boost/container_hash/hash.hpp>
 #include <boost/functional/hash.hpp>
 #include <boost/ptr_container/ptr_vector.hpp>
-#include <boost/range/adaptor/sliced.hpp>
 #include <cassert>
 #include <chrono>
 #include <cstddef>
@@ -22,6 +22,7 @@
 #include <limits>
 #include <memory>
 #include <queue>
+#include <ranges>
 #include <string>
 #include <string_view>
 #include <tuple>
@@ -82,7 +83,7 @@ bool DecoderPrivate::buildLattice(
     LatticeMap &lattice = l.d_ptr->lattice_;
 
     // Create the root node.
-    if (!lattice.count(&graph.start())) {
+    if (!lattice.contains(&graph.start())) {
         lattice[&graph.start()].push_back(
             q->createLatticeNode(graph, model_, "", model_->beginSentence(),
                                  {nullptr, &graph.start()}, state, 0));
@@ -151,7 +152,7 @@ bool DecoderPrivate::buildLattice(
             latticeUnit.push_back(node.release());
         }
     }
-    if (!lattice.count(&graph.end())) {
+    if (!lattice.contains(&graph.end())) {
         return false;
     }
 
@@ -175,8 +176,8 @@ void DecoderPrivate::forwardSearch(
     // forward search
     auto updateForNode = [&](const SegmentGraphBase &,
                              const SegmentGraphNode *graphNode) {
-        if (graphNode == start || !lattice.count(graphNode) ||
-            ignore.count(graphNode)) {
+        if (graphNode == start || !lattice.contains(graphNode) ||
+            ignore.contains(graphNode)) {
             return true;
         }
         auto &latticeNodes = lattice[graphNode];
@@ -207,8 +208,7 @@ void DecoderPrivate::forwardSearch(
                 } else {
                     searchSize = lattice[from].size();
                 }
-                for (auto &parent :
-                     searchFrom | boost::adaptors::sliced(0, searchSize)) {
+                for (auto &parent : searchFrom | std::views::take(searchSize)) {
                     auto score = parent.score() +
                                  model_->score(parent.state(), node, state);
                     if (score > maxScore) {
@@ -270,7 +270,8 @@ void DecoderPrivate::backwardSearch(const SegmentGraph &graph, Lattice &l,
             std::priority_queue<std::shared_ptr<NBestNode>,
                                 std::vector<std::shared_ptr<NBestNode>>,
                                 NBestNodeLess<std::shared_ptr<NBestNode>>>;
-        PriorityQueueType q, result;
+        PriorityQueueType q;
+        PriorityQueueType result;
 
         auto *eos = &lattice[nullptr][0];
         auto newNBestNode = [](const LatticeNode *node) {
@@ -284,7 +285,7 @@ void DecoderPrivate::backwardSearch(const SegmentGraph &graph, Lattice &l,
             q.pop();
             if (bos == node->node_) {
                 auto sentence = concatNBest(node.get());
-                if (dup.count(sentence)) {
+                if (dup.contains(sentence)) {
                     continue;
                 }
 
@@ -308,7 +309,7 @@ void DecoderPrivate::backwardSearch(const SegmentGraph &graph, Lattice &l,
                     searchSize = lattice[node->node_->from()].size();
                 }
                 for (auto &from : lattice[node->node_->from()] |
-                                      boost::adaptors::sliced(0, searchSize)) {
+                                      std::views::take(searchSize)) {
                     auto score =
                         model_->score(from.state(), *node->node_, state) +
                         node->node_->cost();
