@@ -6,9 +6,9 @@
 #ifndef _FCITX_LIBIME_CORE_DICTIONARY_H_
 #define _FCITX_LIBIME_CORE_DICTIONARY_H_
 
-#include "libimecore_export.h"
 #include <functional>
 #include <libime/core/lattice.h>
+#include <libime/core/libimecore_export.h>
 #include <libime/core/segmentgraph.h>
 #include <memory>
 #include <unordered_set>
