@@ -7,9 +7,18 @@
 #include "userlanguagemodel.h"
 #include "constants.h"
 #include "historybigram.h"
-#include "lm/model.hh"
+#include "languagemodel.h"
+#include "lm/state.hh"
 #include "utils_p.h"
+#include <algorithm>
+#include <cassert>
 #include <cmath>
+#include <fcitx-utils/macros.h>
+#include <istream>
+#include <memory>
+#include <ostream>
+#include <string_view>
+#include <utility>
 
 namespace libime {
 
@@ -31,7 +40,7 @@ public:
     }
 
     void setWordToState(State &state, const WordNode *node) const {
-        return storeNative<const WordNode *>(
+        storeNative<const WordNode *>(
             reinterpret_cast<char *>(state.data() + sizeof(lm::ngram::State)),
             node);
     }
@@ -72,7 +81,7 @@ void UserLanguageModel::load(std::istream &in) {
 }
 void UserLanguageModel::save(std::ostream &out) {
     FCITX_D();
-    return d->history_.save(out);
+    d->history_.save(out);
 }
 
 void UserLanguageModel::setHistoryWeight(float w) {
