@@ -6,18 +6,22 @@
 #ifndef _FCITX_LIBIME_PINYIN_PINYINCONTEXT_H_
 #define _FCITX_LIBIME_PINYIN_PINYINCONTEXT_H_
 
-#include "libimepinyin_export.h"
+#include <cstddef>
 #include <fcitx-utils/macros.h>
 #include <libime/core/inputbuffer.h>
+#include <libime/core/languagemodel.h>
 #include <libime/core/lattice.h>
+#include <libime/pinyin/libimepinyin_export.h>
 #include <memory>
+#include <string>
+#include <string_view>
 #include <unordered_set>
+#include <utility>
 #include <vector>
 
 namespace libime {
 class PinyinIME;
 class PinyinContextPrivate;
-class LatticeNode;
 enum class PinyinPreeditMode;
 
 class LIBIMEPINYIN_EXPORT PinyinContext : public InputBuffer {
@@ -58,6 +62,19 @@ public:
     void cancel();
     bool cancelTill(size_t pos);
 
+    /**
+     * Create a custom selection
+     *
+     * This allows Engine to do make a custom selection that is not pinyin.
+     *
+     * @param inputLength the length of characters to match in the input
+     * @param segment segment
+     * @param encodedPinyin whether this segment has a pinyin
+     * @since 1.1.7
+     */
+    void selectCustom(size_t inputLength, std::string_view segment,
+                      std::string_view encodedPinyin = "");
+
     /// Whether the input is fully selected.
     bool selected() const;
 
@@ -90,6 +107,10 @@ public:
     /// Selected hanzi segments.
     std::vector<std::string> selectedWords() const;
 
+    /// Selected hanzi with encoded pinyin
+    std::vector<std::pair<std::string, std::string>>
+    selectedWordsWithPinyin() const;
+
     /// Get the full pinyin string of the selected part.
     std::string selectedFullPinyin() const;
 
