@@ -5,12 +5,18 @@
  */
 
 #include "pinyindata.h"
-#include "libime/pinyin/pinyinencoder.h"
-#include <boost/algorithm/string/predicate.hpp>
+#include "pinyinencoder.h"
+#include <algorithm>
+#include <cstdint>
+#include <fcitx-utils/log.h>
 #include <fcitx-utils/stringutils.h>
+#include <iterator>
+#include <optional>
+#include <string>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -50,7 +56,7 @@ const std::vector<bool> &getEncodedInitialFinal() {
             687, 148, 147, 146, 145, 709, 713, 4,   712, 707, 696, 695, 697,
             565, 569, 570, 568, 563, 552, 547, 545, 544, 540, 692, 691, 689,
             688, 686, 684, 23,  21,  19,  8,   1,   0,   832, 831, 830, 829,
-            828, 230, 20};
+            828, 230, 20,  163, 803, 11};
         a.resize(900);
         std::fill(a.begin(), a.end(), false);
         for (auto i : encodedInitialFinalSet) {
@@ -299,6 +305,11 @@ const PinyinMap &getPinyinMap() {
         {"xi", PinyinInitial::X, PinyinFinal::I, PinyinFuzzyFlag::None},
         {"wu", PinyinInitial::W, PinyinFinal::U, PinyinFuzzyFlag::None},
         {"wo", PinyinInitial::W, PinyinFinal::O, PinyinFuzzyFlag::None},
+        {"wong", PinyinInitial::W, PinyinFinal::ONG, PinyinFuzzyFlag::None},
+        {"won", PinyinInitial::W, PinyinFinal::ONG,
+         PinyinFuzzyFlag::CommonTypo},
+        {"wogn", PinyinInitial::W, PinyinFinal::ONG,
+         PinyinFuzzyFlag::CommonTypo},
         {"wegn", PinyinInitial::W, PinyinFinal::ENG,
          PinyinFuzzyFlag::CommonTypo},
         {"weng", PinyinInitial::W, PinyinFinal::ENG, PinyinFuzzyFlag::None},
@@ -812,7 +823,9 @@ const PinyinMap &getPinyinMap() {
         {"dign", PinyinInitial::D, PinyinFinal::ING,
          PinyinFuzzyFlag::CommonTypo},
         {"ding", PinyinInitial::D, PinyinFinal::ING, PinyinFuzzyFlag::None},
-        {"din", PinyinInitial::D, PinyinFinal::ING, PinyinFuzzyFlag::IN_ING},
+        {"din", PinyinInitial::D, PinyinFinal::IN, PinyinFuzzyFlag::None},
+        {"din", PinyinInitial::D, PinyinFinal::ING,
+         PinyinFuzzyFlag::CommonTypo},
         {"die", PinyinInitial::D, PinyinFinal::IE, PinyinFuzzyFlag::None},
         {"diao", PinyinInitial::D, PinyinFinal::IAO, PinyinFuzzyFlag::None},
         {"diagn",
@@ -903,6 +916,11 @@ const PinyinMap &getPinyinMap() {
         {"cai", PinyinInitial::C, PinyinFinal::AI, PinyinFuzzyFlag::None},
         {"ca", PinyinInitial::C, PinyinFinal::A, PinyinFuzzyFlag::None},
         {"bu", PinyinInitial::B, PinyinFinal::U, PinyinFuzzyFlag::None},
+        {"bogn", PinyinInitial::B, PinyinFinal::ONG,
+         PinyinFuzzyFlag::CommonTypo},
+        {"bong", PinyinInitial::B, PinyinFinal::ONG, PinyinFuzzyFlag::None},
+        {"bon", PinyinInitial::B, PinyinFinal::ONG,
+         PinyinFuzzyFlag::CommonTypo},
         {"bo", PinyinInitial::B, PinyinFinal::O, PinyinFuzzyFlag::None},
         {"bign", PinyinInitial::B, PinyinFinal::ING,
          PinyinFuzzyFlag::CommonTypo},
@@ -940,145 +958,191 @@ const PinyinMap &getPinyinMap() {
     return pinyinMap;
 }
 
+enum class FuzzyUpdatePhase {
+    CommonTypo_UV_JQXY,
+    CommonTypo_ON_ONG,
+    CommonTypo_IN_ING,
+    CommonTypo_Swap_NG_UE_UA_UAN,
+    CommonTypo_Swap_UANG,
+    AdvancedTypo_Swap_XH_UN,
+    AdvancedTypo_Swap_Length2,
+    AdvancedTypo_Swap_Length3,
+    AdvancedTypo_Swap_Length4,
+    AdvancedTypo_Swap_XHY_XYH,
+};
+
+PinyinFuzzyFlag fuzzyPhaseToFlag(FuzzyUpdatePhase phase) {
+    switch (phase) {
+    case FuzzyUpdatePhase::CommonTypo_UV_JQXY:
+    case FuzzyUpdatePhase::CommonTypo_ON_ONG:
+    case FuzzyUpdatePhase::CommonTypo_IN_ING:
+    case FuzzyUpdatePhase::CommonTypo_Swap_NG_UE_UA_UAN:
+    case FuzzyUpdatePhase::CommonTypo_Swap_UANG:
+        return PinyinFuzzyFlag::CommonTypo;
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_XH_UN:
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length2:
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length3:
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length4:
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_XHY_XYH:
+        return PinyinFuzzyFlag::AdvancedTypo;
+    }
+    return PinyinFuzzyFlag::CommonTypo;
+}
+
 std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
-                                      PinyinFuzzyFlag fz, int pass) {
+                                      FuzzyUpdatePhase phase) {
     if (entry.pinyin() == "m" || entry.pinyin() == "n" ||
         entry.pinyin() == "r" || entry.pinyin() == "ng" ||
         entry.pinyin() == "ou") {
         return std::nullopt;
     }
     auto result = entry.pinyin();
-    switch (fz) {
-    case PinyinFuzzyFlag::CommonTypo: {
-        if (pass == 0) {
-            // Allow non standard usage like jv jve jvan jvuang
-            if (result[0] == 'j' || result[0] == 'q' || result[0] == 'x' ||
-                result[0] == 'y') {
-                if (boost::algorithm::ends_with(result, "u") &&
-                    !boost::algorithm::ends_with(result, "iu") &&
-                    !boost::algorithm::ends_with(result, "ou")) {
-                    result.back() = 'v';
-                }
+    const PinyinFuzzyFlag fz = fuzzyPhaseToFlag(phase);
+    switch (phase) {
+    case FuzzyUpdatePhase::CommonTypo_UV_JQXY: {
+        // Allow non standard usage like jv jve jvan jvuang
+        if (result[0] == 'j' || result[0] == 'q' || result[0] == 'x' ||
+            result[0] == 'y') {
+            if (result.ends_with("u") && !result.ends_with("iu") &&
+                !result.ends_with("ou")) {
+                result.back() = 'v';
+            }
 
-                if (boost::algorithm::ends_with(result, "ue")) {
-                    result[result.size() - 2] = 'v';
-                }
-                if (boost::algorithm::ends_with(result, "uan")) {
-                    result[result.size() - 3] = 'v';
-                }
-                if (boost::algorithm::ends_with(result, "uang")) {
-                    result[result.size() - 4] = 'v';
-                }
-            }
-        } else if (pass == 1) {
-            // Allow lon -> long
-            if (boost::algorithm::ends_with(result, "ong")) {
-                result.pop_back();
-            }
-        } else if (pass == 2) {
-            // Allow ying -> yign
-            if (boost::algorithm::ends_with(result, "ng")) {
-                result[result.size() - 2] = 'g';
-                result[result.size() - 1] = 'n';
-            } else if (boost::algorithm::ends_with(result, "ue")) {
-                // Allow fuzzy for uv, that does not cause ambiguity.
-                result[result.size() - 2] = 'e';
-                result[result.size() - 1] = 'u';
-            } else if (boost::algorithm::ends_with(result, "ve")) {
-                result[result.size() - 2] = 'e';
-                result[result.size() - 1] = 'v';
-            } else if (boost::algorithm::ends_with(result, "ua")) {
-                result[result.size() - 2] = 'a';
-                result[result.size() - 1] = 'u';
-            } else if (boost::algorithm::ends_with(result, "uai")) {
-                result[result.size() - 3] = 'a';
-                result[result.size() - 2] = 'u';
-            } else if (boost::algorithm::ends_with(result, "uan")) {
-                result[result.size() - 3] = 'a';
-                result[result.size() - 2] = 'u';
-            } else if (boost::algorithm::ends_with(result, "van")) {
-                result[result.size() - 3] = 'a';
+            if (result.ends_with("ue") || result.ends_with("un")) {
                 result[result.size() - 2] = 'v';
             }
-        } else if (pass == 3) {
-            // this conflicts with "ng" rule, so need a separate pass.
-            if (boost::algorithm::ends_with(result, "uang")) {
-                result[result.size() - 4] = 'a';
-                result[result.size() - 3] = 'u';
-            } else if (boost::algorithm::ends_with(result, "vang")) {
-                result[result.size() - 4] = 'a';
+            if (result.ends_with("uan")) {
                 result[result.size() - 3] = 'v';
             }
+            if (result.ends_with("uang")) {
+                result[result.size() - 4] = 'v';
+            }
+        }
+    } break;
+    case FuzzyUpdatePhase::CommonTypo_ON_ONG:
+        // Allow lon -> long
+        if (result.ends_with("ong")) {
+            result.pop_back();
         }
         break;
-    case PinyinFuzzyFlag::AdvancedTypo:
-        if (pass == 0) {
-            // Allow reversed zhe -> hze
-            if (boost::algorithm::starts_with(result, "zh") ||
-                boost::algorithm::starts_with(result, "sh") ||
-                boost::algorithm::starts_with(result, "ch")) {
-                std::swap(result[0], result[1]);
-            } else if (boost::algorithm::ends_with(result, "un") &&
-                       !boost::algorithm::ends_with(result, "aun")) {
-                result[result.size() - 2] = 'n';
-                result[result.size() - 1] = 'u';
-            }
-        } else if (pass == 1) {
-            if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
-                break;
-            }
-            for (const auto two : {"ai", "ia", "ei", "ie", "ao", "uo", "ou",
-                                   "iu", "an", "en", "in"}) {
-                if (boost::algorithm::ends_with(result, two)) {
-                    std::swap(result[result.size() - 2],
-                              result[result.size() - 1]);
-                }
-            }
-        } else if (pass == 2) {
-            if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
-                break;
-            }
-            for (const auto three :
-                 {"ang", "eng", "ing", "ong", "iao", "ian"}) {
-                if (boost::algorithm::ends_with(result, three)) {
-                    std::swap(result[result.size() - 3],
-                              result[result.size() - 2]);
-                }
-            }
-        } else if (pass == 3) {
-            if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
-                break;
-            }
-            for (const auto four : {"iang", "iong"}) {
-                if (boost::algorithm::ends_with(result, four)) {
-                    std::swap(result[result.size() - 4],
-                              result[result.size() - 3]);
-                }
-            }
-        } else if (pass == 4) {
-            if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
-                break;
-            }
-            // zhe -> zeh.
-            if (result.size() == 3 && result[1] == 'h' &&
-                entry.flags() == PinyinFuzzyFlag::None) {
+    case FuzzyUpdatePhase::CommonTypo_IN_ING:
+        // Allow din -> ding
+        if (result == "ding") {
+            result.pop_back();
+        }
+        break;
+    case FuzzyUpdatePhase::CommonTypo_Swap_NG_UE_UA_UAN:
+        // Allow ying -> yign
+        if (result.ends_with("ng")) {
+            result[result.size() - 2] = 'g';
+            result[result.size() - 1] = 'n';
+        } else if (result.ends_with("ue")) {
+            // Allow fuzzy for uv, that does not cause ambiguity.
+            result[result.size() - 2] = 'e';
+            result[result.size() - 1] = 'u';
+        } else if (result.ends_with("ve")) {
+            result[result.size() - 2] = 'e';
+            result[result.size() - 1] = 'v';
+        } else if (result.ends_with("ua")) {
+            result[result.size() - 2] = 'a';
+            result[result.size() - 1] = 'u';
+        } else if (result.ends_with("uai") || result.ends_with("uan")) {
+            result[result.size() - 3] = 'a';
+            result[result.size() - 2] = 'u';
+        } else if (result.ends_with("van")) {
+            result[result.size() - 3] = 'a';
+            result[result.size() - 2] = 'v';
+        }
+        break;
+    case FuzzyUpdatePhase::CommonTypo_Swap_UANG:
+        // this conflicts with "ng" rule, so need a separate pass.
+        if (result.ends_with("uang")) {
+            result[result.size() - 4] = 'a';
+            result[result.size() - 3] = 'u';
+        } else if (result.ends_with("vang")) {
+            result[result.size() - 4] = 'a';
+            result[result.size() - 3] = 'v';
+        }
+        break;
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_XH_UN:
+        // Allow reversed zhe -> hze
+        if (result.starts_with("zh") || result.starts_with("sh") ||
+            result.starts_with("ch")) {
+            std::swap(result[0], result[1]);
+        } else if (result.ends_with("un") && !result.ends_with("aun")) {
+            result[result.size() - 2] = 'n';
+            result[result.size() - 1] = 'u';
+        }
+        break;
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length2:
+        if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
+            break;
+        }
+        for (const auto *const two : {"ai", "ia", "ei", "ie", "ao", "uo", "ou",
+                                      "iu", "an", "en", "in"}) {
+            if (result.ends_with(two)) {
                 std::swap(result[result.size() - 2], result[result.size() - 1]);
             }
         }
         break;
-    }
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length3:
+        if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
+            break;
+        }
+        for (const auto *const three :
+             {"ang", "eng", "ing", "ong", "iao", "ian"}) {
+            if (result.ends_with(three)) {
+                std::swap(result[result.size() - 3], result[result.size() - 2]);
+            }
+        }
+        break;
 
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_Length4:
+        if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
+            break;
+        }
+        for (const auto *const four : {"iang", "iong"}) {
+            if (result.ends_with(four)) {
+                std::swap(result[result.size() - 4], result[result.size() - 3]);
+            }
+        }
+        break;
+
+    case FuzzyUpdatePhase::AdvancedTypo_Swap_XHY_XYH:
+        if (entry.flags().test(PinyinFuzzyFlag::AdvancedTypo)) {
+            break;
+        }
+        // zhe -> zeh.
+        if (result.size() == 3 && result[1] == 'h' &&
+            entry.flags() == PinyinFuzzyFlag::None) {
+            std::swap(result[result.size() - 2], result[result.size() - 1]);
+        }
+        break;
+    default:
+        break;
+    }
+    if (result == entry.pinyin()) {
+        return std::nullopt;
+    }
+    return PinyinEntry(result.data(), entry.initial(), entry.final(),
+                       entry.flags() | fz);
+}
+
+std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
+                                      PinyinFuzzyFlag fz) {
+    auto result = entry.pinyin();
+    switch (fz) {
     case PinyinFuzzyFlag::VE_UE: {
-        if (boost::algorithm::ends_with(result, "ve")) {
+        if (result.ends_with("ve")) {
             result[result.size() - 2] = 'u';
         }
         break;
     }
 
     case PinyinFuzzyFlag::IAN_IANG: {
-        if (boost::algorithm::ends_with(result, "ian")) {
+        if (result.ends_with("ian")) {
             result.push_back('g');
-        } else if (boost::algorithm::ends_with(result, "iang")) {
+        } else if (result.ends_with("iang")) {
             result.pop_back();
         }
         break;
@@ -1088,55 +1152,52 @@ std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
         if (entry.flags() != PinyinFuzzyFlag::None) {
             break;
         }
-        if (boost::algorithm::ends_with(result, "uan")) {
+        if (result.ends_with("uan")) {
             result.push_back('g');
-        } else if (boost::algorithm::ends_with(result, "uang")) {
+        } else if (result.ends_with("uang")) {
             result.pop_back();
         }
         break;
     }
 
     case PinyinFuzzyFlag::AN_ANG: {
-        if (boost::algorithm::ends_with(result, "uan") ||
-            boost::algorithm::ends_with(result, "uang")) {
+        if (result.ends_with("uan") || result.ends_with("uang")) {
             break;
         }
-        if (boost::algorithm::ends_with(result, "ian") ||
-            boost::algorithm::ends_with(result, "iang")) {
+        if (result.ends_with("ian") || result.ends_with("iang")) {
             break;
         }
-        if (boost::algorithm::ends_with(result, "an")) {
+        if (result.ends_with("an")) {
             result.push_back('g');
-        } else if (boost::algorithm::ends_with(result, "ang")) {
+        } else if (result.ends_with("ang")) {
             result.pop_back();
         }
         break;
     }
 
     case PinyinFuzzyFlag::EN_ENG: {
-        if (boost::algorithm::ends_with(result, "en")) {
+        if (result.ends_with("en")) {
             result.push_back('g');
-        } else if (boost::algorithm::ends_with(result, "eng")) {
+        } else if (result.ends_with("eng")) {
             result.pop_back();
         }
         break;
     }
 
     case PinyinFuzzyFlag::IN_ING: {
-        if (boost::algorithm::ends_with(result, "in")) {
+        if (result.ends_with("in")) {
             result.push_back('g');
-        } else if (boost::algorithm::ends_with(result, "ing")) {
+        } else if (result.ends_with("ing")) {
             result.pop_back();
         }
         break;
     }
 
     case PinyinFuzzyFlag::U_OU: {
-        if (boost::algorithm::ends_with(result, "ou")) {
+        if (result.ends_with("ou")) {
             result.pop_back();
             result.back() = 'u';
-        } else if (boost::algorithm::ends_with(result, "u") &&
-                   !boost::algorithm::ends_with(result, "iu")) {
+        } else if (result.ends_with("u") && !result.ends_with("iu")) {
             result.back() = 'o';
             result.push_back('u');
         }
@@ -1147,9 +1208,9 @@ std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
         if (entry.flags() != PinyinFuzzyFlag::None) {
             break;
         }
-        if (boost::algorithm::starts_with(result, "ch")) {
+        if (result.starts_with("ch")) {
             result.erase(std::next(result.begin()));
-        } else if (boost::algorithm::starts_with(result, "c")) {
+        } else if (result.starts_with("c")) {
             result.insert(std::next(result.begin()), 'h');
         }
         break;
@@ -1159,9 +1220,9 @@ std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
         if (entry.flags() != PinyinFuzzyFlag::None) {
             break;
         }
-        if (boost::algorithm::starts_with(result, "sh")) {
+        if (result.starts_with("sh")) {
             result.erase(std::next(result.begin()));
-        } else if (boost::algorithm::starts_with(result, "s")) {
+        } else if (result.starts_with("s")) {
             result.insert(std::next(result.begin()), 'h');
         }
         break;
@@ -1171,27 +1232,36 @@ std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
         if (entry.flags() != PinyinFuzzyFlag::None) {
             break;
         }
-        if (boost::algorithm::starts_with(result, "zh")) {
+        if (result.starts_with("zh")) {
             result.erase(std::next(result.begin()));
-        } else if (boost::algorithm::starts_with(result, "z")) {
+        } else if (result.starts_with("z")) {
             result.insert(std::next(result.begin()), 'h');
         }
         break;
     }
 
     case PinyinFuzzyFlag::F_H: {
-        if (boost::algorithm::starts_with(result, "f")) {
+        if (result.starts_with("f")) {
             result.front() = 'h';
-        } else if (boost::algorithm::starts_with(result, "h")) {
+        } else if (result.starts_with("h")) {
             result.front() = 'f';
         }
         break;
     }
 
     case PinyinFuzzyFlag::L_N: {
-        if (boost::algorithm::starts_with(result, "l")) {
+        if (result.starts_with("l")) {
             result.front() = 'n';
-        } else if (boost::algorithm::starts_with(result, "n")) {
+        } else if (result.starts_with("n")) {
+            result.front() = 'l';
+        }
+        break;
+    }
+
+    case PinyinFuzzyFlag::L_R: {
+        if (result.starts_with("l")) {
+            result.front() = 'r';
+        } else if (result.starts_with("r")) {
             result.front() = 'l';
         }
         break;
@@ -1206,17 +1276,24 @@ std::optional<PinyinEntry> applyFuzzy(const PinyinEntry &entry,
                        entry.flags() | fz);
 }
 
-void applyFuzzy(PinyinMap &map, PinyinFuzzyFlag fz, int pass = 0) {
+template <typename T>
+void applyFuzzyToMap(PinyinMap &map, T fuzzy) {
     std::vector<PinyinEntry> newEntries;
-    for (auto &entry : map) {
-        if (auto newEntry = applyFuzzy(entry, fz, pass)) {
+    for (const auto &entry : map) {
+        if (auto newEntry = applyFuzzy(entry, fuzzy)) {
             newEntries.push_back(*newEntry);
         }
     }
 
     for (const auto &newEntry : newEntries) {
         if (auto iter = map.find(newEntry.pinyin()); iter != map.end()) {
-            if (iter->flags() == PinyinFuzzyFlag::None) {
+            bool force = false;
+            if constexpr (std::is_same_v<T, FuzzyUpdatePhase>) {
+                if (fuzzy == FuzzyUpdatePhase::CommonTypo_IN_ING) {
+                    force = true;
+                }
+            }
+            if (!force && iter->flags() == PinyinFuzzyFlag::None) {
                 continue;
             }
         }
@@ -1240,18 +1317,25 @@ const PinyinMap &getPinyinMapV2() {
                         PinyinFuzzyFlag::UAN_UANG, PinyinFuzzyFlag::IAN_IANG,
                         PinyinFuzzyFlag::VE_UE, PinyinFuzzyFlag::F_H,
                         PinyinFuzzyFlag::L_N, PinyinFuzzyFlag::Z_ZH,
-                        PinyinFuzzyFlag::S_SH, PinyinFuzzyFlag::C_CH}) {
-            applyFuzzy(filtered, fz);
+                        PinyinFuzzyFlag::S_SH, PinyinFuzzyFlag::C_CH,
+                        PinyinFuzzyFlag::L_R}) {
+            applyFuzzyToMap(filtered, fz);
+        }
+
+        for (auto phase : {
+                 FuzzyUpdatePhase::CommonTypo_UV_JQXY,
+                 FuzzyUpdatePhase::CommonTypo_ON_ONG,
+                 FuzzyUpdatePhase::CommonTypo_IN_ING,
+                 FuzzyUpdatePhase::CommonTypo_Swap_NG_UE_UA_UAN,
+                 FuzzyUpdatePhase::CommonTypo_Swap_UANG,
+                 FuzzyUpdatePhase::AdvancedTypo_Swap_XH_UN,
+                 FuzzyUpdatePhase::AdvancedTypo_Swap_Length2,
+                 FuzzyUpdatePhase::AdvancedTypo_Swap_Length3,
+                 FuzzyUpdatePhase::AdvancedTypo_Swap_Length4,
+                 FuzzyUpdatePhase::AdvancedTypo_Swap_XHY_XYH,
+             }) {
+            applyFuzzyToMap(filtered, phase);
         }
-        applyFuzzy(filtered, PinyinFuzzyFlag::CommonTypo, 0);
-        applyFuzzy(filtered, PinyinFuzzyFlag::CommonTypo, 1);
-        applyFuzzy(filtered, PinyinFuzzyFlag::CommonTypo, 2);
-        applyFuzzy(filtered, PinyinFuzzyFlag::CommonTypo, 3);
-        applyFuzzy(filtered, PinyinFuzzyFlag::AdvancedTypo, 0);
-        applyFuzzy(filtered, PinyinFuzzyFlag::AdvancedTypo, 1);
-        applyFuzzy(filtered, PinyinFuzzyFlag::AdvancedTypo, 2);
-        applyFuzzy(filtered, PinyinFuzzyFlag::AdvancedTypo, 3);
-        applyFuzzy(filtered, PinyinFuzzyFlag::AdvancedTypo, 4);
         return filtered;
     }();
     return map;
