@@ -6,17 +6,19 @@
 #ifndef _FCITX_LIBIME_PINYIN_PINYINDECODER_P_H_
 #define _FCITX_LIBIME_PINYIN_PINYINDECODER_P_H_
 
-#include "pinyindecoder.h"
 #include <libime/core/lattice.h>
+#include <string>
+#include <string_view>
 
 namespace libime {
 
 class PinyinLatticeNodePrivate : public LatticeNodeData {
 public:
-    PinyinLatticeNodePrivate(std::string_view encodedPinyin)
-        : encodedPinyin_(encodedPinyin) {}
+    PinyinLatticeNodePrivate(std::string_view encodedPinyin, bool isCorrection)
+        : encodedPinyin_(encodedPinyin), isCorrection_(isCorrection) {}
 
     std::string encodedPinyin_;
+    bool isCorrection_ = false;
 };
 } // namespace libime
 
