@@ -4,19 +4,32 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "pinyinencoder.h"
+#include "libime/core/segmentgraph.h"
 #include "pinyincorrectionprofile.h"
 #include "pinyindata.h"
 #include "shuangpinprofile.h"
 #include <algorithm>
-#include <boost/algorithm/string.hpp>
+#include <array>
 #include <boost/bimap.hpp>
 #include <boost/container/static_vector.hpp>
+#include <cassert>
+#include <cstddef>
+#include <cstdint>
 #include <fcitx-utils/charutils.h>
+#include <fcitx-utils/log.h>
+#include <fcitx-utils/stringutils.h>
 #include <functional>
+#include <initializer_list>
+#include <iterator>
 #include <queue>
+#include <ranges>
+#include <stdexcept>
+#include <string>
 #include <string_view>
 #include <tuple>
 #include <unordered_map>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -101,7 +114,7 @@ bool hasMatchInMap(const PinyinMap &map, std::string_view range,
     auto iterPair = map.equal_range(range);
     if (iterPair.first != iterPair.second) {
         for (const auto &item :
-             boost::make_iterator_range(iterPair.first, iterPair.second)) {
+             std::ranges::subrange(iterPair.first, iterPair.second)) {
             if (flags.test(item.flags())) {
                 // do not consider m/n/r as complete pinyin
                 return true;
@@ -266,22 +279,33 @@ PinyinEncoder::parseUserPinyin(std::string userPinyin,
                 }
 
                 for (size_t i = 0; i < nNextSize; i++) {
-                    if ((nextSize[i] >= 4 &&
-                         fuzzyFlags.test(PinyinFuzzyFlag::Inner)) ||
-                        (nextSize[i] == 3 &&
-                         flags.test(PinyinFuzzyFlag::InnerShort))) {
+                    auto nextPinyin = str.substr(0, nextSize[i]);
+                    if (nextPinyin == "din" || nextPinyin == "bon" ||
+                        nextPinyin == "won") {
+                        result.addNext(top, top + 2);
+                        result.addNext(top + 2, top + 3);
+                    } else if (nextPinyin == "bong" || nextPinyin == "wong") {
+                        // Do a split like {b,w}o n g
+                        result.addNext(top, top + 2);
+                        result.addNext(top + 2, top + 4);
+                        result.addNext(top + 2, top + 3);
+                        // Skip top+3 -> top+4, in case g can have a better
+                        q.push(top + 3);
+                    } else if ((nextPinyin.size() >= 4 &&
+                                fuzzyFlags.test(PinyinFuzzyFlag::Inner)) ||
+                               (nextPinyin.size() == 3 &&
+                                flags.test(PinyinFuzzyFlag::InnerShort))) {
                         const auto &innerSegments = getInnerSegment();
-                        auto iter = innerSegments.find(
-                            std::string{str.substr(0, nextSize[i])});
+                        auto iter = innerSegments.find(std::string(nextPinyin));
                         if (iter != innerSegments.end()) {
                             result.addNext(top,
                                            top + iter->second.first.size());
                             result.addNext(top + iter->second.first.size(),
                                            top + nextSize[i]);
                         }
-                    } else if (nextSize[i] == 2 &&
+                    } else if (nextPinyin.size() == 2 &&
                                flags.test(PinyinFuzzyFlag::InnerShort) &&
-                               str.substr(0, 2) == "ng") {
+                               nextPinyin == "ng") {
                         // Handle ng -> n'g, the condition is so simple so we
                         // don't make it go through the inner segment lookup.
                         result.addNext(top, top + 1);
@@ -382,8 +406,7 @@ std::vector<char> PinyinEncoder::encodeFullPinyin(std::string_view pinyin) {
 std::vector<char>
 PinyinEncoder::encodeFullPinyinWithFlags(std::string_view pinyin,
                                          PinyinFuzzyFlags flags) {
-    std::vector<std::string> pinyins;
-    boost::split(pinyins, pinyin, boost::is_any_of("'"));
+    std::vector<std::string> pinyins = fcitx::stringutils::split(pinyin, "'");
     std::vector<char> result;
     result.resize(pinyins.size() * 2);
     int idx = 0;
@@ -466,7 +489,7 @@ std::string PinyinEncoder::decodeFullPinyin(const char *data, size_t size) {
             result += '\'';
         }
         result += initialToString(static_cast<PinyinInitial>(data[i * 2]));
-        result += finalToString(static_cast<PinyinFinal>(data[i * 2 + 1]));
+        result += finalToString(static_cast<PinyinFinal>(data[(i * 2) + 1]));
     }
     return result;
 }
@@ -592,6 +615,7 @@ void getFuzzy(FuzzyPinyinSyllables<FuzzyValue> &syls, PinyinSyllable syl,
             {PinyinInitial::Z, PinyinInitial::ZH, PinyinFuzzyFlag::Z_ZH},
             {PinyinInitial::F, PinyinInitial::H, PinyinFuzzyFlag::F_H},
             {PinyinInitial::L, PinyinInitial::N, PinyinFuzzyFlag::L_N},
+            {PinyinInitial::L, PinyinInitial::R, PinyinFuzzyFlag::L_R},
         };
 
     for (const auto &initialFuzzy : initialFuzzies) {
@@ -698,7 +722,7 @@ stringToSyllablesImpl(std::string_view pinyinView, const PinyinMap &map,
     if (pinyin != "m" && pinyin != "n" && pinyin != "r") {
         auto iterPair = map.equal_range(pinyin);
         for (const auto &item :
-             boost::make_iterator_range(iterPair.first, iterPair.second)) {
+             std::ranges::subrange(iterPair.first, iterPair.second)) {
             if (flags.test(item.flags())) {
                 getFuzzy(result, {item.initial(), item.final()}, flags,
                          /*isSp=*/false,
