@@ -6,13 +6,22 @@
 #ifndef _FCITX_LIBIME_PINYIN_PINYINMATCHSTATE_P_H_
 #define _FCITX_LIBIME_PINYIN_PINYINMATCHSTATE_P_H_
 
+#include "libime/core/languagemodel.h"
+#include "libime/core/segmentgraph.h"
+#include <boost/container_hash/hash.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <fcitx-utils/macros.h>
 #include <libime/core/lattice.h>
 #include <libime/core/lrucache.h>
 #include <libime/pinyin/pinyindictionary.h>
 #include <libime/pinyin/pinyinmatchstate.h>
 #include <memory>
+#include <string>
+#include <string_view>
 #include <unordered_map>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -87,25 +96,23 @@ public:
 };
 
 // A list of all search paths
-typedef std::vector<MatchedPinyinPath> MatchedPinyinPaths;
+using MatchedPinyinPaths = std::vector<MatchedPinyinPath>;
 
 // Map from SegmentGraphNode to Search Paths.
-typedef std::unordered_map<const SegmentGraphNode *, MatchedPinyinPaths>
-    NodeToMatchedPinyinPathsMap;
+using NodeToMatchedPinyinPathsMap =
+    std::unordered_map<const SegmentGraphNode *, MatchedPinyinPaths>;
 
 // A cache for all PinyinTries. From a pinyin string to its matched
 // PinyinTrieNode
-typedef std::unordered_map<
+using PinyinTrieNodeCache = std::unordered_map<
     const PinyinTrie *,
     LRUCache<std::string, std::shared_ptr<MatchedPinyinTrieNodes>,
-             PinyinStringHasher>>
-    PinyinTrieNodeCache;
+             PinyinStringHasher>>;
 
 // A cache for PinyinMatchResult.
-typedef std::unordered_map<
+using PinyinMatchResultCache = std::unordered_map<
     const PinyinTrie *,
-    LRUCache<std::string, std::vector<PinyinMatchResult>, PinyinStringHasher>>
-    PinyinMatchResultCache;
+    LRUCache<std::string, std::vector<PinyinMatchResult>, PinyinStringHasher>>;
 
 class PinyinMatchStatePrivate {
 public:
