@@ -6,9 +6,17 @@
 #ifndef _FCITX_LIBIME_PINYIN_PREDICTION_H_
 #define _FCITX_LIBIME_PINYIN_PREDICTION_H_
 
-#include "libime/pinyin/pinyindictionary.h"
-#include "libimepinyin_export.h"
+#include <cstddef>
+#include <fcitx-utils/macros.h>
+#include <libime/core/languagemodel.h>
 #include <libime/core/prediction.h>
+#include <libime/pinyin/libimepinyin_export.h>
+#include <libime/pinyin/pinyindictionary.h>
+#include <memory>
+#include <string>
+#include <string_view>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
