@@ -8,7 +8,11 @@
 #include "autophrasedict.h"
 #include "constants.h"
 #include "libime/core/datrie.h"
+#include "libime/core/dictionary.h"
+#include "libime/core/languagemodel.h"
 #include "libime/core/lattice.h"
+#include "libime/core/segmentgraph.h"
+#include "libime/core/utils_p.h"
 #include "libime/core/zstdfilter.h"
 #include "log.h"
 #include "tablebaseddictionary_p.h"
@@ -16,23 +20,32 @@
 #include "tableoptions.h"
 #include "tablerule.h"
 #include <algorithm>
-#include <boost/algorithm/string.hpp>
-#include <boost/iostreams/filtering_streambuf.hpp>
-#include <boost/range/adaptor/reversed.hpp>
 #include <cassert>
 #include <chrono>
 #include <cstdint>
+#include <cstdlib>
 #include <cstring>
+#include <exception>
 #include <fcitx-utils/log.h>
+#include <fcitx-utils/macros.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx-utils/utf8.h>
 #include <fstream>
+#include <ios>
+#include <iostream>
 #include <istream>
+#include <iterator>
+#include <memory>
+#include <optional>
+#include <ostream>
+#include <ranges>
 #include <set>
 #include <stdexcept>
 #include <string>
 #include <string_view>
+#include <tuple>
 #include <unordered_set>
+#include <utility>
 #include <vector>
 
 namespace libime {
@@ -65,7 +78,7 @@ enum {
 
 enum class BuildPhase { PhaseConfig, PhaseRule, PhaseData, PhasePhrase };
 
-const char *strConst[2][STR_LAST] = {
+std::string_view strConst[2][STR_LAST] = {
     {"键码=", "码长=", "规避字符=", "拼音=", "拼音长度=", "[数据]",
      "[组词规则]", "提示=", "构词=", "[词组]"},
     {"KeyCode=", "Length=", "InvalidChar=", "Pinyin=", "PinyinLength=",
@@ -73,7 +86,6 @@ const char *strConst[2][STR_LAST] = {
 
 constexpr std::string_view UserDictAutoMark = "[Auto]";
 constexpr std::string_view UserDictDeleteMark = "[Delete]";
-constexpr std::string_view ExtraDictPhraseMark = "[Phrase]";
 
 // A better version of key + keyValueSeparator + value. It tries to avoid
 // multiple allocation.
@@ -157,7 +169,7 @@ void saveTrieToText(const DATrie<uint32_t> &trie, std::ostream &out) {
     });
     for (auto &item : temp) {
         out << std::get<0>(item) << " " << maybeEscapeValue(std::get<1>(item))
-            << std::endl;
+            << '\n';
     }
 }
 
@@ -165,9 +177,7 @@ uint32_t maxValue(const DATrie<uint32_t> &trie) {
     uint32_t max = 0;
     trie.foreach(
         [&max](uint32_t value, size_t, DATrie<uint32_t>::position_type) {
-            if (value + 1 > max) {
-                max = value + 1;
-            }
+            max = std::max(value + 1, max);
             return true;
         });
     return max;
@@ -279,7 +289,7 @@ bool TableBasedDictionaryPrivate::matchTrie(
                     auto curPos = position;
                     auto strCode = fcitx::utf8::UCS4ToUTF8(code);
                     auto result = trie.traverse(strCode, curPos);
-                    if (!trie.isNoPath(result)) {
+                    if (!DATrie<unsigned int>::isNoPath(result)) {
                         newPositions.push_back(curPos);
                     }
                 }
@@ -290,7 +300,7 @@ bool TableBasedDictionaryPrivate::matchTrie(
                     std::distance(charRange.first, charRange.second));
                 auto curPos = position;
                 auto result = trie.traverse(chr, curPos);
-                if (!trie.isNoPath(result)) {
+                if (!DATrie<unsigned int>::isNoPath(result)) {
                     newPositions.push_back(curPos);
                 }
             }
@@ -376,13 +386,13 @@ bool TableBasedDictionaryPrivate::validate() const {
     if (inputCode_.empty()) {
         return false;
     }
-    if (inputCode_.count(pinyinKey_)) {
+    if (inputCode_.contains(pinyinKey_)) {
         return false;
     }
-    if (inputCode_.count(promptKey_)) {
+    if (inputCode_.contains(promptKey_)) {
         return false;
     }
-    if (inputCode_.count(phraseKey_)) {
+    if (inputCode_.contains(phraseKey_)) {
         return false;
     }
     return true;
@@ -393,11 +403,11 @@ TableBasedDictionaryPrivate::parseDataLine(std::string_view buf, bool user) {
     uint32_t special[3] = {pinyinKey_, phraseKey_, promptKey_};
     PhraseFlag specialFlag[] = {PhraseFlag::Pinyin, PhraseFlag::ConstructPhrase,
                                 PhraseFlag::Prompt};
-    auto spacePos = buf.find_first_of(" \n\r\f\v\t");
+    auto spacePos = buf.find_first_of(FCITX_WHITESPACE);
     if (spacePos == std::string::npos || spacePos + 1 == buf.size()) {
         return {};
     }
-    auto wordPos = buf.find_first_not_of(" \n\r\f\v\t", spacePos);
+    auto wordPos = buf.find_first_not_of(FCITX_WHITESPACE, spacePos);
     if (spacePos == std::string::npos || spacePos + 1 == buf.size()) {
         return {};
     }
@@ -411,7 +421,7 @@ TableBasedDictionaryPrivate::parseDataLine(std::string_view buf, bool user) {
     }
 
     uint32_t firstChar;
-    const auto *next =
+    std::string_view::iterator next =
         fcitx::utf8::getNextChar(key.begin(), key.end(), &firstChar);
     auto *iter = std::find(std::begin(special), std::end(special), firstChar);
     PhraseFlag flag = user ? PhraseFlag::User : PhraseFlag::None;
@@ -638,19 +648,16 @@ void TableBasedDictionary::loadText(std::istream &in) {
 
     std::string buf;
 
-    auto check_option = [&buf](int index) {
-        if (buf.compare(0, std::strlen(strConst[0][index]),
-                        strConst[0][index]) == 0) {
-            return 0;
+    auto consumeOptionPrefix = [](std::string_view &buf, int index) {
+        if (fcitx::stringutils::consumePrefix(buf, strConst[0][index])) {
+            return true;
         }
-        if (buf.compare(0, std::strlen(strConst[1][index]),
-                        strConst[1][index]) == 0) {
-            return 1;
+        if (fcitx::stringutils::consumePrefix(buf, strConst[1][index])) {
+            return true;
         }
-        return -1;
+        return false;
     };
 
-    auto isSpaceCheck = boost::is_any_of(" \n\t\r\v\f");
     auto phase = BuildPhase::PhaseConfig;
     while (!in.eof()) {
         if (!std::getline(in, buf)) {
@@ -662,55 +669,57 @@ void TableBasedDictionary::loadText(std::istream &in) {
             continue;
         }
 
-        boost::trim_if(buf, isSpaceCheck);
+        auto line = fcitx::stringutils::trimView(buf);
 
         switch (phase) {
         case BuildPhase::PhaseConfig: {
-            if (buf[0] == '#') {
+            if (fcitx::stringutils::startsWith(line, "#")) {
                 continue;
             }
 
-            int match = -1;
-            if ((match = check_option(STR_KEYCODE)) >= 0) {
-                const std::string code =
-                    buf.substr(strlen(strConst[match][STR_KEYCODE]));
-                auto range = fcitx::utf8::MakeUTF8CharRange(code);
+            if (consumeOptionPrefix(line, STR_KEYCODE)) {
+                auto range = fcitx::utf8::MakeUTF8CharRange(line);
                 d->inputCode_ = std::set<uint32_t>(range.begin(), range.end());
-            } else if ((match = check_option(STR_CODELEN)) >= 0) {
-                d->codeLength_ =
-                    std::stoi(buf.substr(strlen(strConst[match][STR_CODELEN])));
-            } else if (check_option(STR_PINYINLEN) >= 0) {
+            } else if (consumeOptionPrefix(line, STR_CODELEN)) {
+                d->codeLength_ = std::stoi(std::string(line));
+            } else if (consumeOptionPrefix(line, STR_PINYINLEN)) {
                 // Deprecated option.
-            } else if ((match = check_option(STR_IGNORECHAR)) >= 0) {
-                const std::string ignoreChars =
-                    buf.substr(strlen(strConst[match][STR_IGNORECHAR]));
-                auto range = fcitx::utf8::MakeUTF8CharRange(ignoreChars);
+            } else if (consumeOptionPrefix(line, STR_IGNORECHAR)) {
+                auto range = fcitx::utf8::MakeUTF8CharRange(line);
                 d->ignoreChars_ =
                     std::set<uint32_t>(range.begin(), range.end());
-            } else if ((match = check_option(STR_PINYIN)) >= 0) {
-                d->pinyinKey_ = buf[strlen(strConst[match][STR_PINYIN])];
-            } else if ((match = check_option(STR_PROMPT)) >= 0) {
-                d->promptKey_ = buf[strlen(strConst[match][STR_PROMPT])];
-            } else if ((match = check_option(STR_CONSTRUCTPHRASE)) >= 0) {
-                d->phraseKey_ =
-                    buf[strlen(strConst[match][STR_CONSTRUCTPHRASE])];
-            } else if (check_option(STR_DATA) >= 0) {
+            } else if (consumeOptionPrefix(line, STR_PINYIN)) {
+                const auto chr = fcitx::utf8::getChar(line);
+                if (fcitx::utf8::isValidChar(chr)) {
+                    d->pinyinKey_ = chr;
+                }
+            } else if (consumeOptionPrefix(line, STR_PROMPT)) {
+                const auto chr = fcitx::utf8::getChar(line);
+                if (fcitx::utf8::isValidChar(chr)) {
+                    d->promptKey_ = chr;
+                }
+            } else if (consumeOptionPrefix(line, STR_CONSTRUCTPHRASE)) {
+                const auto chr = fcitx::utf8::getChar(line);
+                if (fcitx::utf8::isValidChar(chr)) {
+                    d->phraseKey_ = chr;
+                }
+            } else if (consumeOptionPrefix(line, STR_DATA)) {
                 phase = BuildPhase::PhaseData;
                 if (!d->validate()) {
                     throw std::invalid_argument("file format is invalid");
                 }
                 break;
-            } else if (check_option(STR_RULE) >= 0) {
+            } else if (consumeOptionPrefix(line, STR_RULE)) {
                 phase = BuildPhase::PhaseRule;
                 break;
             }
             break;
         }
         case BuildPhase::PhaseRule: {
-            if (buf[0] == '#') {
+            if (fcitx::stringutils::startsWith(line, "#")) {
                 continue;
             }
-            if (check_option(STR_DATA) >= 0) {
+            if (consumeOptionPrefix(line, STR_DATA)) {
                 phase = BuildPhase::PhaseData;
                 if (!d->validate()) {
                     throw std::invalid_argument("file format is invalid");
@@ -718,28 +727,31 @@ void TableBasedDictionary::loadText(std::istream &in) {
                 break;
             }
 
-            if (buf.empty()) {
+            if (line.empty()) {
                 continue;
             }
 
-            d->rules_.emplace_back(buf, d->codeLength_);
+            d->rules_.emplace_back(std::string(line), d->codeLength_);
             break;
         }
         case BuildPhase::PhaseData:
-            if (check_option(STR_PHRASE) >= 0) {
+            if (consumeOptionPrefix(line, STR_PHRASE)) {
                 phase = BuildPhase::PhasePhrase;
                 if (!hasRule()) {
                     throw std::invalid_argument(
                         "file has phrase section but no rule");
                 }
+                break;
             }
-            d->insertDataLine(buf, false);
+            d->insertDataLine(line, false);
             break;
-        case BuildPhase::PhasePhrase:
-            maybeUnescapeValue(buf);
-            insert(buf, PhraseFlag::None);
+        case BuildPhase::PhasePhrase: {
+            std::string phrase{line};
+            maybeUnescapeValue(phrase);
+            insert(phrase, PhraseFlag::None);
             break;
         }
+        }
     }
 
     if (phase != BuildPhase::PhaseData && phase != BuildPhase::PhasePhrase) {
@@ -754,35 +766,35 @@ void TableBasedDictionary::saveText(std::ostream &out) {
     for (auto c : d->inputCode_) {
         out << fcitx::utf8::UCS4ToUTF8(c);
     }
-    out << std::endl;
-    out << strConst[1][STR_CODELEN] << d->codeLength_ << std::endl;
+    out << '\n';
+    out << strConst[1][STR_CODELEN] << d->codeLength_ << '\n';
     if (!d->ignoreChars_.empty()) {
         out << strConst[1][STR_IGNORECHAR];
         for (auto c : d->ignoreChars_) {
             out << fcitx::utf8::UCS4ToUTF8(c);
         }
-        out << std::endl;
+        out << '\n';
     }
     if (d->pinyinKey_) {
         out << strConst[1][STR_PINYIN] << fcitx::utf8::UCS4ToUTF8(d->pinyinKey_)
-            << std::endl;
+            << '\n';
     }
     if (d->promptKey_) {
         out << strConst[1][STR_PROMPT] << fcitx::utf8::UCS4ToUTF8(d->promptKey_)
-            << std::endl;
+            << '\n';
     }
     if (d->phraseKey_) {
         out << strConst[1][STR_CONSTRUCTPHRASE]
-            << fcitx::utf8::UCS4ToUTF8(d->phraseKey_) << std::endl;
+            << fcitx::utf8::UCS4ToUTF8(d->phraseKey_) << '\n';
     }
 
     if (hasRule()) {
-        out << strConst[1][STR_RULE] << std::endl;
+        out << strConst[1][STR_RULE] << '\n';
         for (const auto &rule : d->rules_) {
-            out << rule.toString() << std::endl;
+            out << rule.toString() << '\n';
         }
     }
-    out << strConst[1][STR_DATA] << std::endl;
+    out << strConst[1][STR_DATA] << '\n';
     std::string buf;
     if (d->promptKey_) {
         auto promptString = fcitx::utf8::UCS4ToUTF8(d->promptKey_);
@@ -796,7 +808,7 @@ void TableBasedDictionary::saveText(std::ostream &out) {
                 }
                 std::string_view ref(buf);
                 out << promptString << ref.substr(sep + 1) << " "
-                    << maybeEscapeValue(ref.substr(0, sep)) << std::endl;
+                    << maybeEscapeValue(ref.substr(0, sep)) << '\n';
                 return true;
             });
     }
@@ -812,7 +824,7 @@ void TableBasedDictionary::saveText(std::ostream &out) {
                 }
                 std::string_view ref(buf);
                 out << phraseString << ref.substr(sep + 1) << " "
-                    << maybeEscapeValue(ref.substr(0, sep)) << std::endl;
+                    << maybeEscapeValue(ref.substr(0, sep)) << '\n';
                 return true;
             });
     }
@@ -907,7 +919,8 @@ void TableBasedDictionary::loadUser(const char *filename, TableFormat format) {
 
 void TableBasedDictionary::loadUser(std::istream &in, TableFormat format) {
     FCITX_D();
-    uint32_t magic = 0, version = 0;
+    uint32_t magic = 0;
+    uint32_t version = 0;
     switch (format) {
     case TableFormat::Binary:
         throw_if_io_fail(unmarshall(in, magic));
@@ -930,35 +943,35 @@ void TableBasedDictionary::loadUser(std::istream &in, TableFormat format) {
         }
         break;
     case TableFormat::Text: {
-        std::string buf;
-        auto isSpaceCheck = boost::is_any_of(" \n\t\r\v\f");
+        std::string lineBuf;
         enum class UserDictState { Phrase, Auto, Delete };
 
         UserDictState state = UserDictState::Phrase;
         while (!in.eof()) {
-            if (!std::getline(in, buf)) {
+            if (!std::getline(in, lineBuf)) {
                 break;
             }
 
             // Validate everything first, so it's easier to process.
-            if (!fcitx::utf8::validate(buf)) {
+            if (!fcitx::utf8::validate(lineBuf)) {
                 continue;
             }
-            boost::trim_if(buf, isSpaceCheck);
-            if (buf == UserDictAutoMark) {
+            auto line = fcitx::stringutils::trimView(lineBuf);
+            if (line == UserDictAutoMark) {
                 state = UserDictState::Auto;
                 continue;
-            } else if (buf == UserDictDeleteMark) {
+            }
+            if (line == UserDictDeleteMark) {
                 state = UserDictState::Delete;
                 continue;
             }
 
             switch (state) {
             case UserDictState::Phrase:
-                d->insertDataLine(buf, true);
+                d->insertDataLine(line, true);
                 break;
             case UserDictState::Auto: {
-                auto tokens = fcitx::stringutils::split(buf, " \n\t\r\v\f");
+                auto tokens = fcitx::stringutils::split(line, FCITX_WHITESPACE);
                 if (tokens.size() != 3 || !isAllInputCode(tokens[0])) {
                     continue;
                 }
@@ -972,7 +985,7 @@ void TableBasedDictionary::loadUser(std::istream &in, TableFormat format) {
                 }
             } break;
             case UserDictState::Delete: {
-                if (auto data = d->parseDataLine(buf, true)) {
+                if (auto data = d->parseDataLine(line, true)) {
                     auto &[key, value, flag] = *data;
                     auto entry = generateTableEntry(key, value);
                     d->deletionTrie_.set(entry, 0);
@@ -987,10 +1000,10 @@ void TableBasedDictionary::loadUser(std::istream &in, TableFormat format) {
     }
 }
 
-void TableBasedDictionary::saveUser(const char *filename, TableFormat) {
+void TableBasedDictionary::saveUser(const char *filename, TableFormat format) {
     std::ofstream fout(filename, std::ios::out | std::ios::binary);
     throw_if_io_fail(fout);
-    saveUser(fout);
+    saveUser(fout, format);
 }
 
 void TableBasedDictionary::saveUser(std::ostream &out, TableFormat format) {
@@ -1014,7 +1027,7 @@ void TableBasedDictionary::saveUser(std::ostream &out, TableFormat format) {
         saveTrieToText(d->userTrie_, out);
 
         if (!d->autoPhraseDict_.empty()) {
-            out << UserDictAutoMark << std::endl;
+            out << UserDictAutoMark << '\n';
             std::vector<std::tuple<std::string, std::string, int32_t>>
                 autoEntries;
             d->autoPhraseDict_.search(
@@ -1024,13 +1037,13 @@ void TableBasedDictionary::saveUser(std::ostream &out, TableFormat format) {
                                              entry.substr(sep + 1), hit);
                     return true;
                 });
-            for (auto &t : autoEntries | boost::adaptors::reversed) {
+            for (auto &t : autoEntries | std::views::reverse) {
                 out << std::get<0>(t) << " " << maybeEscapeValue(std::get<1>(t))
-                    << " " << std::get<2>(t) << std::endl;
+                    << " " << std::get<2>(t) << '\n';
             }
         }
         if (!d->deletionTrie_.empty()) {
-            out << UserDictDeleteMark << std::endl;
+            out << UserDictDeleteMark << '\n';
             saveTrieToText(d->deletionTrie_, out);
         }
         break;
@@ -1051,7 +1064,8 @@ size_t TableBasedDictionary::loadExtra(std::istream &in, TableFormat format) {
     FCITX_D();
     DATrie<uint32_t> trie;
     uint32_t index = 0;
-    uint32_t magic = 0, version = 0;
+    uint32_t magic = 0;
+    uint32_t version = 0;
     switch (format) {
     case TableFormat::Binary:
         throw_if_io_fail(unmarshall(in, magic));
@@ -1071,31 +1085,32 @@ size_t TableBasedDictionary::loadExtra(std::istream &in, TableFormat format) {
         }
         break;
     case TableFormat::Text: {
-        std::string buf;
-        auto isSpaceCheck = boost::is_any_of(" \n\t\r\v\f");
+        std::string lineBuf;
         enum class ExtraDictState { Data, Phrase };
 
         ExtraDictState state = ExtraDictState::Data;
         while (!in.eof()) {
-            if (!std::getline(in, buf)) {
+            if (!std::getline(in, lineBuf)) {
                 break;
             }
 
             // Validate everything first, so it's easier to process.
-            if (!fcitx::utf8::validate(buf)) {
+            if (!fcitx::utf8::validate(lineBuf)) {
                 continue;
             }
-            boost::trim_if(buf, isSpaceCheck);
-            if (buf == ExtraDictPhraseMark) {
+            const auto line = fcitx::stringutils::trimView(lineBuf);
+            if (line == strConst[0][STR_PHRASE] ||
+                line == strConst[1][STR_PHRASE]) {
                 state = ExtraDictState::Phrase;
                 continue;
             }
 
-            std::string key, value;
+            std::string key;
+            std::string value;
             PhraseFlag flag;
             switch (state) {
             case ExtraDictState::Data:
-                if (auto data = d->parseDataLine(buf, false)) {
+                if (auto data = d->parseDataLine(line, false)) {
                     std::tie(key, value, flag) = *data;
                     if (flag != PhraseFlag::None) {
                         continue;
@@ -1104,7 +1119,7 @@ size_t TableBasedDictionary::loadExtra(std::istream &in, TableFormat format) {
                 }
                 break;
             case ExtraDictState::Phrase:
-                value = buf;
+                value = line;
                 maybeUnescapeValue(value);
                 if (!generate(value, key)) {
                     continue;
@@ -1223,7 +1238,7 @@ bool TableBasedDictionary::insert(std::string_view key, std::string_view value,
         }
 
         if (flag == PhraseFlag::None && fcitx::utf8::length(value) == 1 &&
-            !d->ignoreChars_.count(fcitx::utf8::getChar(value))) {
+            !d->ignoreChars_.contains(fcitx::utf8::getChar(value))) {
             updateReverseLookupEntry(d->singleCharTrie_, key, value, nullptr);
 
             if (hasRule() && !d->phraseKey_) {
@@ -1270,7 +1285,7 @@ bool TableBasedDictionary::generate(std::string_view value,
 }
 
 bool TableBasedDictionary::generateWithHint(
-    std::string_view value, const std::vector<std::string> &hints_,
+    std::string_view value, const std::vector<std::string> &codeHints,
     std::string &key) const {
     FCITX_D();
     if (!hasRule() || value.empty()) {
@@ -1283,26 +1298,28 @@ bool TableBasedDictionary::generateWithHint(
         return false;
     }
 
-    for (const auto &code : hints_) {
+    for (const auto &code : codeHints) {
         if (!fcitx::utf8::validate(code)) {
             return false;
         }
     }
 
-    auto hints = hints_;
+    auto hints = codeHints;
     hints.resize(valueLen);
 
     std::string newKey;
     for (const auto &rule : d->rules_) {
         // check rule can be applied
-        if (!((rule.flag() == TableRuleFlag::LengthEqual &&
-               valueLen == rule.phraseLength()) ||
-              (rule.flag() == TableRuleFlag::LengthLongerThan &&
-               valueLen >= rule.phraseLength()))) {
+        const bool canApplyRule =
+            ((rule.flag() == TableRuleFlag::LengthEqual &&
+              valueLen == rule.phraseLength()) ||
+             (rule.flag() == TableRuleFlag::LengthLongerThan &&
+              valueLen >= rule.phraseLength()));
+        if (!canApplyRule) {
             continue;
         }
 
-        auto hints = hints_;
+        auto hints = codeHints;
         hints.resize(valueLen);
         // Fill hints first.
         if (!d->validateHints(hints, rule)) {
@@ -1330,7 +1347,7 @@ bool TableBasedDictionary::generateWithHint(
                 index = valueLen - ruleEntry.character();
             }
             iter = fcitx::utf8::nextNChar(value.begin(), index);
-            const auto *prev = iter;
+            std::string_view::iterator prev = iter;
             iter = fcitx::utf8::nextChar(iter);
             std::string_view chr(&*prev, std::distance(prev, iter));
 
@@ -1363,7 +1380,7 @@ bool TableBasedDictionary::generateWithHint(
             // Avoid same code being referenced twice.
             // This helps for the case like: p11 and p1z point to the same code
             // character.
-            if (usedChar.count(charIndex)) {
+            if (usedChar.contains(charIndex)) {
                 continue;
             }
             usedChar.insert(charIndex);
@@ -1388,8 +1405,8 @@ bool TableBasedDictionary::isInputCode(uint32_t c) const {
 }
 
 bool TableBasedDictionary::isAllInputCode(std::string_view code) const {
-    const auto *iter = code.begin();
-    const auto *end = code.end();
+    std::string_view::iterator iter = code.begin();
+    std::string_view::iterator end = code.end();
     while (iter != end) {
         uint32_t chr;
         iter = fcitx::utf8::getNextChar(iter, end, &chr);
@@ -1402,18 +1419,17 @@ bool TableBasedDictionary::isAllInputCode(std::string_view code) const {
 
 bool TableBasedDictionary::isEndKey(uint32_t c) const {
     FCITX_D();
-    return !!d->options_.endKey().count(c);
+    return d->options_.endKey().contains(c);
 }
 
 void TableBasedDictionary::statistic() const {
     FCITX_D();
-    std::cout << "Phrase Trie: " << d->phraseTrie_.mem_size() << std::endl
-              << "Single Char Trie: " << d->singleCharTrie_.mem_size()
-              << std::endl
+    std::cout << "Phrase Trie: " << d->phraseTrie_.mem_size() << '\n'
+              << "Single Char Trie: " << d->singleCharTrie_.mem_size() << '\n'
               << "Single char const trie: "
               << d->singleCharConstTrie_.mem_size() << " + "
-              << d->singleCharLookupTrie_.mem_size() << std::endl
-              << "Prompt Trie: " << d->promptTrie_.mem_size() << std::endl;
+              << d->singleCharLookupTrie_.mem_size() << '\n'
+              << "Prompt Trie: " << d->promptTrie_.mem_size() << '\n';
 }
 
 void TableBasedDictionary::setTableOptions(TableOptions option) {
@@ -1594,7 +1610,8 @@ std::string TableBasedDictionary::hint(std::string_view key) const {
 
 void TableBasedDictionary::matchPrefixImpl(
     const SegmentGraph &graph, const GraphMatchCallback &callback,
-    const std::unordered_set<const SegmentGraphNode *> &ignore, void *) const {
+    const std::unordered_set<const SegmentGraphNode *> &ignore,
+    void * /*helper*/) const {
     FCITX_D();
     auto range = fcitx::utf8::MakeUTF8CharRange(graph.data());
     auto hasWildcard =
@@ -1610,7 +1627,7 @@ void TableBasedDictionary::matchPrefixImpl(
     graph.bfs(&graph.start(), [this, &ignore, &path, &callback, hasWildcard,
                                mode](const SegmentGraphBase &graph,
                                      const SegmentGraphNode *node) {
-        if (!node->prevSize() || ignore.count(node)) {
+        if (!node->prevSize() || ignore.contains(node)) {
             return true;
         }
         for (const auto &prev : node->prevs()) {
