@@ -5,22 +5,36 @@
  */
 #include "tablecontext.h"
 #include "constants.h"
-#include "libime/core/decoder.h"
 #include "libime/core/historybigram.h"
+#include "libime/core/inputbuffer.h"
+#include "libime/core/languagemodel.h"
+#include "libime/core/lattice.h"
 #include "libime/core/segmentgraph.h"
 #include "libime/core/userlanguagemodel.h"
-#include "libime/core/utils.h"
+#include "libime/table/tablebaseddictionary.h"
 #include "log.h"
 #include "tablebaseddictionary_p.h"
 #include "tabledecoder.h"
 #include "tableoptions.h"
-#include "tablerule.h"
-#include <boost/ptr_container/ptr_vector.hpp>
+#include <algorithm>
+#include <cassert>
 #include <chrono>
+#include <cstddef>
+#include <cstdint>
+#include <fcitx-utils/inputbuffer.h>
 #include <fcitx-utils/log.h>
-#include <fcitx-utils/stringutils.h>
+#include <fcitx-utils/macros.h>
 #include <fcitx-utils/utf8.h>
+#include <iterator>
+#include <limits>
+#include <memory>
 #include <regex>
+#include <string>
+#include <string_view>
+#include <tuple>
+#include <unordered_map>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -32,6 +46,15 @@ size_t sentenceCodeLength(const SentenceResult &sentence) {
     return node->codeLength();
 }
 
+/// Helper function compare length. If limit is less than 0, it means no
+/// limit. Avoid unsigned / signed compare.
+bool lengthLessThanLimit(size_t length, int limit) {
+    if (limit < 0) {
+        return false;
+    }
+    return length < static_cast<size_t>(limit);
+}
+
 template <OrderPolicy policy>
 struct TableCandidateCompare {
     TableCandidateCompare(int noSortInputLength, bool sortByCodeLength)
@@ -181,7 +204,7 @@ public:
             return false;
         }
         FCITX_Q();
-        return q->code(candidates_[0]) == q->currentCode() &&
+        return libime::TableContext::code(candidates_[0]) == q->currentCode() &&
                (!dict_.tableOptions().exactMatch() ||
                 dict_.hasOneMatchingWord(q->currentCode()));
     };
@@ -525,7 +548,8 @@ void TableContext::update() {
         };
 
         auto &graph = d->graph_;
-        auto bos = &graph.start(), eos = &graph.end();
+        const SegmentGraphNode *bos = &graph.start();
+        const SegmentGraphNode *eos = &graph.end();
         constexpr float pinyinPenalty = -0.5;
         for (const auto &latticeNode : d->lattice_.nodes(eos)) {
             if (latticeNode.from() == bos && latticeNode.to() == eos) {
@@ -539,9 +563,7 @@ void TableContext::update() {
 
         float min = 0;
         for (const auto &cand : d->candidates_) {
-            if (min > cand.score()) {
-                min = cand.score();
-            }
+            min = std::min(min, cand.score());
         }
 
         // FIXME: add an option.
@@ -557,7 +579,7 @@ void TableContext::update() {
             }
             // Check the limit, or if there's no candidate.
             if (min - score < minDistance || candidates().empty()) {
-                insertCandidate(sentence);
+                insertCandidate(std::move(sentence));
             }
         }
         t1 = std::chrono::high_resolution_clock::now();
@@ -622,8 +644,7 @@ void TableContext::update() {
 
 TableContext::CandidateRange TableContext::candidates() const {
     FCITX_D();
-    return boost::make_iterator_range(d->candidates_.begin(),
-                                      d->candidates_.end());
+    return d->candidates_;
 }
 
 size_t TableContext::selectedLength() const {
@@ -771,7 +792,7 @@ void TableContext::learnLast() {
 }
 
 void TableContext::learnAutoPhrase(std::string_view history) {
-    return learnAutoPhrase(history, {});
+    learnAutoPhrase(history, {});
 }
 
 void TableContext::learnAutoPhrase(std::string_view history,
