@@ -4,11 +4,25 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
+#include "libime/table/tabledecoder.h"
+#include "libime/core/languagemodel.h"
+#include "libime/core/lattice.h"
+#include "libime/core/segmentgraph.h"
+#include "libime/table/tablebaseddictionary.h"
 #include "tabledecoder_p.h"
 #include "tableoptions.h"
 #include "tablerule.h"
-#include <boost/range/adaptor/filtered.hpp>
+#include <algorithm>
+#include <cstddef>
+#include <cstdint>
 #include <fcitx-utils/utf8.h>
+#include <iterator>
+#include <memory>
+#include <ranges>
+#include <string>
+#include <string_view>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -23,7 +37,7 @@ bool checkRuleCanBeUsedAsAutoRule(const TableRule &rule) {
         return false;
     }
 
-    auto range = rule.entries() | boost::adaptors::filtered(isNotPlaceHolder);
+    auto range = rule.entries() | std::views::filter(isNotPlaceHolder);
     auto iter = std::begin(range);
     auto end = std::end(range);
     int currentChar = 1;
@@ -55,7 +69,7 @@ bool checkRuleCanBeUsedAsAutoRule(const TableRule &rule) {
 } // namespace
 
 uint32_t TableLatticeNode::index() const {
-    return d_ptr ? d_ptr->index_ : 0xFFFFFFFFu;
+    return d_ptr ? d_ptr->index_ : 0xFFFFFFFFU;
 }
 
 PhraseFlag TableLatticeNode::flag() const {
@@ -87,9 +101,10 @@ TableLatticeNode::TableLatticeNode(
 TableLatticeNode::~TableLatticeNode() = default;
 
 LatticeNode *TableDecoder::createLatticeNodeImpl(
-    const SegmentGraphBase &, const LanguageModelBase *, std::string_view word,
-    WordIndex idx, SegmentGraphPath path, const State &state, float cost,
-    std::unique_ptr<LatticeNodeData> data, bool) const {
+    const SegmentGraphBase & /*graph*/, const LanguageModelBase * /*model*/,
+    std::string_view word, WordIndex idx, SegmentGraphPath path,
+    const State &state, float cost, std::unique_ptr<LatticeNodeData> data,
+    bool /*onlyPath*/) const {
     std::unique_ptr<TableLatticeNodePrivate> tableData(
         static_cast<TableLatticeNodePrivate *>(data.release()));
     return new TableLatticeNode(word, idx, std::move(path), state, cost,
@@ -97,12 +112,12 @@ LatticeNode *TableDecoder::createLatticeNodeImpl(
 }
 
 bool TableDecoder::needSort(const SegmentGraph &graph,
-                            const SegmentGraphNode *) const {
+                            const SegmentGraphNode * /*node*/) const {
     return graph.start().nextSize() != 1;
 }
 
-LIBIMETABLE_EXPORT SegmentGraph graphForCode(std::string_view s,
-                                             const TableBasedDictionary &dict) {
+SegmentGraph graphForCode(std::string_view s,
+                          const TableBasedDictionary &dict) {
     SegmentGraph graph{std::string(s)};
     if (s.empty()) {
         return graph;
@@ -121,8 +136,7 @@ LIBIMETABLE_EXPORT SegmentGraph graphForCode(std::string_view s,
 
             std::vector<int> charSizes(rule->phraseLength());
             for (const auto &entry :
-                 rule->entries() |
-                     boost::adaptors::filtered(isNotPlaceHolder)) {
+                 rule->entries() | std::views::filter(isNotPlaceHolder)) {
                 auto &charSize = charSizes[entry.character() - 1];
                 charSize = std::max(charSize, entry.index());
             }
